package eu.mathiasl.legacychatfix.config;

import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.HashMap;
import java.util.Map;

/**
 * Adapted from: https://git.fngnation.net/never2nv/NVus_Prison_Lite/src/branch/main/src/main/java/me/nvus/nvus_prison_setup/Configs/ConfigManager.java
 */
public class ConfigManager {
    private final JavaPlugin plugin;
    private final Map<String, FileConfiguration> configs;
    private final Map<String, File> configFiles;

    public ConfigManager(JavaPlugin plugin) {
        this.plugin = plugin;
        this.configs = new HashMap<>();
        this.configFiles = new HashMap<>();
    }

    public void reloadConfig(String configName) {
        File configFile = getConfigFile(configName);
        FileConfiguration config = YamlConfiguration.loadConfiguration(configFile);
        configs.put(configName, config);
    }

    public FileConfiguration getConfig(String configName) {
        if (!configs.containsKey(configName)) {
            reloadConfig(configName);
        }
        return configs.get(configName);
    }

    public void saveConfig(String configName) {
        FileConfiguration config = getConfig(configName);
        File configFile = getConfigFile(configName);
        try {
            config.save(configFile);
        } catch (IOException e) {
            plugin.getLogger().severe("Could not save " + configName + " to " + configFile);
        }
    }

    public void saveDefaultConfig(String configName) {
        File configFile = getConfigFile(configName);
        if (!configFile.exists()) {
            plugin.saveResource(configName, false);
            plugin.getLogger().info(configName + " has been created.");
        }
    }

    private File getConfigFile(String configName) {
        if (!configFiles.containsKey(configName)) {
            File configFile = new File(plugin.getDataFolder(), configName);
            configFiles.put(configName, configFile);
        }
        return configFiles.get(configName);
    }


    public void checkAndUpdateConfig(String configName) {
        File configFile = new File(plugin.getDataFolder(), configName);
        if (!configFile.exists()) {
            plugin.saveResource(configName, false);
            return;
        }

        FileConfiguration config = YamlConfiguration.loadConfiguration(configFile);
        String configVersion = config.getString("ConfigVersion", "0"); // Default to "0" if not found
        if (!"1.0".equals(configVersion)) {
            backupAndReplaceConfig(configName);

            // After backing up and copying new ones, you might want to reload these configs into your ConfigManager
            // assuming you have such a method in your ConfigManager
            reloadConfig(configName);
        }
    }

    private void backupAndReplaceConfig(String fileName) {
        File configFile = new File(plugin.getDataFolder(), fileName);
        File backupFile = new File(plugin.getDataFolder(), fileName + ".BACKUP");

        // Backup the old file
        try {
            Files.move(configFile.toPath(), backupFile.toPath());
        } catch (IOException e) {
            plugin.getLogger().warning("Could not backup " + fileName + ": " + e.getMessage());
        }

        // Copy new file from resources
        plugin.saveResource(fileName, false);
    }
}
